@@ -4,7 +4,7 @@ use strict;
 extends 'GOBO::Node';
 
 has type => (is=>'rw', isa=>'GOBO::ClassNode', coerce=>1);
-has supporting_entities => (is=>'rw', isa=>'ArrayRef[GOBO::Node]', coerece=>1);
+has supporting_entities => (is=>'rw', isa=>'ArrayRef[GOBO::Node]');
 
 sub with_str {
     return join('|',@{shift->supporting_entities || []});
