@@ -7,6 +7,9 @@ use Carp;
 use GOBO::Graph;
 use GOBO::Parsers::OBOParser;
 use GOBO::Writers::OBOWriter;
+#use GOBO::Parsers::QuikOBOParser;
+#use GOBO::Parsers::QuikGAFParser;
+use GOBO::Writers::OBOWriter;
 use GOBO::InferenceEngine;
 
 use Data::Dumper;
@@ -151,12 +154,41 @@ sub check_options {
 }
 
 
+sub get_graph {
+	my $args = shift;
+	my $options = $args->{options};
+
+	my $parser;
+	if ($options->{quick_parse})
+	{	# parse the input file and check we get a graph
+		$parser = new GOBO::Parsers::QuikOBOParser(file => $options->{input}, options => $options->{parser_options});
+	}
+	else
+	{	# parse the input file and check we get a graph
+		$parser = new GOBO::Parsers::OBOParser(file => $options->{input});
+	}
+	$parser->parse;
+	die "Error: parser could not find a graph in " . $options->{input} . "!\n" unless $parser->graph;
+	print STDERR "Finished parsing file " . $options->{input} . "\n" if $options->{verbose};
+
+	return $parser->graph;
+
+}
+
+
 =head2 get_subset_nodes
 
 get the subset nodes we want by whatever means, fair or foul
+roots will be added to the subset after determining that there are other terms
+in the subset
 
 input:  graph   => Graph object
         options => option_h
+          options may be:
+          get_all_subsets => 1
+          subset => { subset_name => 1, subset_2_name => 1 }
+          
+          # subset_regexp => regular expression
 
 output: data hash or death with an appropriate error
         data hash will be of the form
@@ -170,7 +202,9 @@ sub get_subset_nodes {
 	my $graph = $args->{graph};
 	my $options = $args->{options};
 
-	confess( (caller(0))[3] . ": missing required arguments. Dying" ) unless $graph && $options;
+	confess( (caller(0))[3] . ": missing required arguments. Dying" ) unless defined $graph && $options;
+
+#	print STDERR "options: " . Dumper($options) . "\n";
 
 	my $data_h;
 
@@ -246,6 +280,7 @@ sub get_subset_nodes {
 		}
 	}
 	
+	
 	# merge the subsets into one if we want combined results
 	if ($options->{combined})
 	{	my @subs = keys %{$data_h->{subset}};
@@ -257,6 +292,14 @@ sub get_subset_nodes {
 		} @subs;
 	}
 
+
+	## add the roots to the subsets (??)
+	foreach my $r (keys %{$data_h->{roots}})
+	{	foreach my $s (keys %{$data_h->{subset}})
+		{	$data_h->{subset}{$s}{$r}++;
+		}
+	}
+
 	return $data_h;
 }
 
@@ -299,9 +342,11 @@ sub get_graph_relations {
 
 =head2 get_graph_links
 
-input:  input   => hash of input nodes in the form node_id => 1
-        roots   => root nodes in the form node_id => 1)
-        subset  => subset nodes in the form node_id => 1)
+input:  input   => hash of input nodes in the form node_id => 1; optional; uses
+                   all graph nodes if not specified
+        subset  => subset nodes in the form node_id => 1; optional; uses
+                   all graph nodes if not specified
+#        roots   => root nodes in the form node_id => 1
         graph   => Graph object
         inf_eng => inference engine (a new one will be created if not)
         
@@ -312,42 +357,63 @@ output: node data in the form
 
 sub get_graph_links {
 	my $args = shift;
-	my $roots = $args->{roots};
+#	my $roots = $args->{roots};
+	my $graph = $args->{graph};
 	my $subset = $args->{subset};
 	my $input = $args->{input};
-	my $graph = $args->{graph};
-	my $ie = $args->{inf_eng} || new GOBO::InferenceEngine(graph => $graph);
+	my $ie = $args->{inf_eng};
 
-	confess( (caller(0))[3] . "get_graph_links: missing required arguments" ) unless $roots && $subset && $input && $graph && $ie;
+	confess( (caller(0))[3] . ": missing required arguments" ) unless $graph;
+
+	# no input: use all the terms in the graph as input
+	if (! $input )
+	{	$input->{$_->id} = 1 foreach (@{$graph->terms});
+	}
+
+	$ie = new GOBO::InferenceEngine(graph => $graph) if ! $ie;
+
+	confess( (caller(0))[3] . ": missing required arguments" ) unless $graph && $ie && $input;
 
 	# get rid of any existing data
 	my $node_data;
 
-	# get all the links between nodes in the subset or subset nodes and root
-	foreach my $t (sort keys %$input)
-	{
-		## asserted links
-		foreach (@{ $graph->get_outgoing_links($t) })
-		{
-			# skip it unless the target is a root or in the subset
-			next unless $subset->{$_->target->id} || $roots->{$_->target->id} ;
-
-			$node_data->{graph}{$t}{$_->relation->id}{$_->target->id} = 1;
-		}
-
-		foreach (@{ $ie->get_inferred_target_links($t) })
-		{
-			# skip it unless the target is a root or in the subset
-			next unless $subset->{$_->target->id} || $roots->{$_->target->id} ;
-
-			# skip it if we already have this link
-			next if defined #$data->{nodes}{graph}{$t}{$_->target->id} &&
-			$node_data->{graph}{$t}{$_->relation->id}{$_->target->id};
-
-			## add to a list of inferred entries
-			$node_data->{graph}{$t}{$_->relation->id}{$_->target->id} = 2;
+	if ($subset)
+	{	# get all the links between the input nodes and those in the subset
+		foreach my $t (keys %$input)
+		{	## asserted links
+			foreach (@{ $graph->get_outgoing_links($t) })
+			{	# skip it unless the target is a root or in the subset
+				next unless $subset->{$_->target->id}; # || $roots->{$_->target->id} ;
+				$node_data->{graph}{$t}{$_->relation->id}{$_->target->id} = 1;
+			}
+	
+			foreach (@{ $ie->get_inferred_target_links($t) })
+			{	# skip it unless the target is a root or in the subset
+				next unless $subset->{$_->target->id}; # || $roots->{$_->target->id} ;
+				# skip it if we already have this link
+				next if defined $node_data->{graph}{$t}{$_->relation->id}{$_->target->id};
+				## add to a list of inferred entries
+				$node_data->{graph}{$t}{$_->relation->id}{$_->target->id} = 2;
+			}
 		}
 	}
+	else
+	{	# get all the links involving the input nodes
+		foreach my $t (keys %$input)
+		{	## asserted links
+			foreach (@{ $graph->get_outgoing_links($t) })
+			{	$node_data->{graph}{$t}{$_->relation->id}{$_->target->id} = 1;
+			}
+	
+			foreach (@{ $ie->get_inferred_target_links($t) })
+			{	# skip it if we already have this link
+				next if defined $node_data->{graph}{$t}{$_->relation->id}{$_->target->id};
+				## add to a list of inferred entries
+				$node_data->{graph}{$t}{$_->relation->id}{$_->target->id} = 2;
+			}
+		}
+	}
+
 	return $node_data;
 }
 
@@ -400,6 +466,9 @@ sub remove_redundant_relationships {
 		## slim down the relationships
 		## get rid of redundant relations
 		# these are the closest to the root
+		
+		## this could probably be done more effectively / efficiently
+		
 		foreach my $r (keys %{$slimmed->{target_node_rel}})
 		{	foreach my $r2 (keys %{$slimmed->{target_node_rel}{$r}})
 			{	# if both exist...
@@ -439,7 +508,7 @@ input:  graph_data => data hash with nodes and relations specified as
           plus various rearrangements, with a hash key specifying the ordering
           e.g. {node_target_rel}
                {target_node_rel}
-        options => option_h
+        options => option_h  # no options specified as yet
 
 output: new data hash, slimmed down, with relations specified as
                {graph}{ node_id }{ relation_id }{ target_id }
@@ -516,112 +585,6 @@ sub trim_graph {
 }
 
 
-=head2 add_all_relations_to_graph
-
-input:  old_g => old Graph object
-        new_g => new Graph object (created if does not exist)
-
-output: new graph with relations from the old graph added
-
-=cut
-
-sub add_all_relations_to_graph {
-	my $args = shift;
-	my $old_g = $args->{old_g};
-	my $new_g = $args->{new_g} || new GOBO::Graph; # if ! $new_g;
-
-	confess( (caller(0))[3] . ": missing required argument old_g. Dying" ) unless $old_g && defined $new_g;
-
-	# add all the relations from the other graph
-	foreach (@{$old_g->relations})
-	{	$new_g->add_relation($old_g->noderef($_)) unless $_->id eq 'is_a';
-
-		if ($old_g->get_outgoing_links($_))
-		{	foreach (@{$old_g->get_outgoing_links($_)})
-			{
-				$new_g->add_link( new GOBO::LinkStatement(
-					node => $old_g->noderef($_->node),
-					relation => $old_g->noderef($_->relation),
-					target => $old_g->noderef($_->target)
-				) );
-			}
-		}
-	}
-	return $new_g;
-}
-
-
-=head2 add_extra_stuff_to_graph
-
-input:  old_g => old Graph object
-        new_g => new Graph object (created if does not exist)
-
-output: new graph with various attributes from the old graph added 
-
-=cut
-
-sub add_extra_stuff_to_graph {
-	my $args = shift;
-	my $old_g = $args->{old_g};
-	my $new_g = $args->{new_g};
-
-	$new_g = new GOBO::Graph if ! $new_g;
-
-	confess( (caller(0))[3] . ": missing required arguments. Dying" ) unless $old_g && defined $new_g;
-
-	foreach my $attrib qw( version source date comment declared_subsets property_value_map )
-	{	$new_g->$attrib( $old_g->$attrib ) if $old_g->$attrib;
-	}
-	return $new_g;
-}
-
-
-=head2 add_nodes_and_links_to_graph
-
-input:  graph_data => data hash with nodes and relations specified as
-             { node_id }{ relation_id }{ target_id }
-        old_g      => old graph, containing nodes and relations specified in the data hash
-        new_g      => preferably the new graph, containing relations
-                      (created if does not exist)
-
-output: new graph, containing all the nodes and relations specified in 
-
-=cut
-
-sub add_nodes_and_links_to_graph {
-	my $args = shift;
-	my $graph_data = $args->{graph_data};
-	my $old_g = $args->{old_g};
-	my $new_g = $args->{new_g};
-
-	$new_g = new GOBO::Graph if ! $new_g;
-
-	confess( (caller(0))[3] . ": missing required arguments. Dying" ) unless $old_g && defined $new_g && $graph_data;
-
-	## add the relations to the graph if absent
-	if (! $new_g->relations )
-	{	$new_g = add_all_relations_to_graph({ old_g => $old_g, new_g => $new_g });
-	}
-
-	# add the nodes to the graph
-	foreach my $n ( keys %$graph_data )
-	{	# add the nodes to the graph
-		$new_g->add_term( $old_g->noderef( $n ) ) if ! $new_g->get_term($n);
-
-		foreach my $r ( keys %{$graph_data->{$n}} )
-		{	foreach my $t ( keys %{$graph_data->{$n}{$r}} )
-			{	$new_g->add_term( $old_g->noderef( $t ) ) if ! $new_g->get_term($t);
-				$new_g->add_link( new GOBO::LinkStatement(
-					node => $new_g->noderef($n),
-					relation => $new_g->noderef($r),
-					target => $new_g->noderef($t)
-				) );
-			}
-		}
-	}
-	return $new_g;
-}
-
 
 =head2 populate_lookup_hashes
 
@@ -815,7 +778,7 @@ input:  graph_data => data hash with nodes and relations specified as
                {target_node_rel}
         id        => id of node to find the closest ancestral node of
         relation  => relation id, if wanted
-        options => option_h
+        options   => option_h
 
 output: new data hash, slimmed down, with relations specified as
                {graph}{ node_id }{ relation_id }{ target_id }
@@ -871,6 +834,7 @@ sub get_furthest_ancestral_nodes {
 	}
 
 	#TODO: add in a check for the root nodes
+	
 
 
 	my $new_d;
@@ -890,12 +854,6 @@ sub get_furthest_ancestral_nodes {
 				{	#	b is node, a is target
 					#	forget about b, look at the next in the list
 					next;
-=cut
-					#	forget about a, go on to the next list item
-					push @list_by_rel, $b;
-					push @list_by_rel, @list2_by_rel if @list2_by_rel;
-					next REL_SLIMDOWN_LOOP;
-=cut
 				}
 				elsif ($d->{node_target_rel}{$a}{$b})
 				{	#	a is node, b is target
@@ -903,10 +861,6 @@ sub get_furthest_ancestral_nodes {
 					push @list_by_rel, $b;
 					push @list_by_rel, @list2_by_rel if @list2_by_rel;
 					next REL_SLIMDOWN_LOOP;
-=cut
-					#	forget about b, look at the next in the list
-					next;
-=cut
 				}
 				else
 				{	#a and b aren't related
@@ -930,4 +884,283 @@ sub get_furthest_ancestral_nodes {
 }
 
 
+=head2 topological_sort
+
+input:  graph_data => data hash with nodes and relations specified as
+               {graph}{ node_id }{ relation_id }{ target_id }
+               nb: must already have had all that reasoning stuff done
+          plus various rearrangements, with a hash key specifying the ordering
+          e.g. {node_target_rel}
+               {target_node_rel}
+        id        => id of node to do the topological sort on
+        relation  => relation id, if wanted
+        options   => option_h
+
+output: topo sorted list
+
+For a given term, finds the furthest node[s]
+
+=cut
+
+sub topological_sort {
+	my $args = shift;
+	my $d = $args->{graph_data};
+	my $id = $args->{id};
+#	my $rel_wanted = $args->{relation} || undef;
+	my $options = $args->{options};
+
+	confess( (caller(0))[3] . ": missing required arguments. Dying" ) unless $id && values %{$d->{graph}};
+
+	if (! $d->{graph}{$id})
+	{	print STDERR "$id is a root node... sob!\n";
+		return;
+	}
+	
+	my @sorted;   # Empty list that will contain the sorted nodes
+	my @leafy;    # Set of all nodes with no incoming edges
+=algorithm:
+while leafy is non-empty do
+	remove a node n from leafy
+	insert n into sorted
+	for each node m with an edge e from n to m do
+		remove edge e from the graph
+		if m has no other incoming edges then
+			insert m into leafy
+if graph has edges then
+	output error message (graph has at least one cycle)
+else 
+	output message (proposed topologically sorted order: sorted)
+=cut
+
+#	print STDERR "input data: " . Dumper($d);
+
+	@leafy = ( $id );
+	my $graph;  # this will be all the nodes related to $id
+
+	foreach my $r (keys %{$d->{graph}{$id}})
+	{	foreach (keys %{$d->{graph}{$id}{$r}})
+		{	$graph->{node_target}{$id}{$_} = 1;
+			$graph->{target_node}{$_}{$id} = 1;
+		}
+	}
+
+	## get all relations involving terms listed as keys in $graph->{target_node}
+	foreach my $t (keys %{$graph->{target_node}})
+	{	if ($d->{graph}{$t})
+		{	foreach my $r (keys %{$d->{graph}{$t}})
+			{	foreach (keys %{$d->{graph}{$t}{$r}})
+				{	if ($graph->{target_node}{$_})
+					{	$graph->{node_target}{$t}{$_} = 1;
+						$graph->{target_node}{$_}{$t} = 1;
+					}
+				}
+			}
+		}
+	}
+
+	while (@leafy)
+	{	my $n = pop @leafy;
+		push @sorted, $n;
+		if (defined $graph->{node_target}{$n} && defined values %{$graph->{node_target}{$n}})
+		{	foreach my $m (keys %{$graph->{node_target}{$n}})
+			{	undef $graph->{node_target}{$n}{$m};
+				undef $graph->{target_node}{$m}{$n};
+
+				my $none;
+				if (defined $graph->{target_node}{$m} && defined values %{$graph->{target_node}{$m}})
+				{	foreach (keys %{$graph->{target_node}{$m}})
+					{	if (defined $graph->{target_node}{$m}{$_})
+						{	$none++;
+							last;
+						}
+					}
+					if (! $none )
+					{	push @leafy, $m;
+					}
+				}
+			}
+		}
+	}
+
+	return [ @sorted ];
+
+}
+
+
+## Graph cloning stuff ##
+
+
+=head2 add_all_relations_to_graph
+
+input:  old_g => old Graph object
+        new_g => new Graph object (created if does not exist)
+        no_rel_links => 1  if links should be NOT added (default is to add them)
+
+output: new graph with relations from the old graph added
+
+=cut
+
+sub add_all_relations_to_graph {
+	my $args = shift;
+	my $old_g = $args->{old_g};
+	my $new_g = $args->{new_g} || new GOBO::Graph; # if ! $new_g;
+	my $no_rel_links = $args->{no_rel_links} || undef;
+
+	confess( (caller(0))[3] . ": missing required argument old_g. Dying" ) unless $old_g && defined $new_g;
+
+	sub check_for_relation {
+		my ($graph, $r) = @_;
+		return 1 if $r->id ne 'is_a' && ! $graph->get_relation($r);
+		return;
+	}
+
+	if ($no_rel_links)
+	{	$new_g->add_relation($old_g->noderef($_)) foreach @{$old_g->relations};
+	}
+	else
+	{	# add all the relations from the other graph
+		foreach (@{$old_g->relations})
+		{	$new_g->add_relation($old_g->noderef($_)) if check_for_relation($new_g, $_);
+	
+			if ($old_g->get_outgoing_links($_))
+			{	foreach (@{$old_g->get_outgoing_links($_)})
+				{	
+					$new_g->add_relation( $old_g->noderef( $_->relation ) ) if check_for_relation($new_g, $_->relation);
+					$new_g->add_relation( $old_g->noderef( $_->target ) ) if check_for_relation($new_g, $_->target);
+					
+					$new_g->add_link( new GOBO::LinkStatement(
+						node => $new_g->noderef($_->node),
+						relation => $new_g->noderef($_->relation),
+						target => $new_g->noderef($_->target)
+					) );
+				}
+			}
+		}
+	}
+
+	return $new_g;
+}
+
+
+=head2 add_all_terms_to_graph
+
+input:  old_g => old Graph object
+        new_g => new Graph object (created if does not exist)
+        no_term_links => 1  if links between terms should NOT be added 
+                            (default is to add them)
+        no_rel_links  => 1  if links between relations should NOT be added
+                            (default is to add them; only matters if no_term_links
+                             has been specified)
+
+output: new graph with terms from the old graph added
+
+=cut
+
+sub add_all_terms_to_graph {
+	my $args = shift;
+	my $old_g = $args->{old_g};
+	my $new_g = $args->{new_g} || new GOBO::Graph; # if ! $new_g;
+	my $no_term_links = $args->{no_term_links} || undef;
+
+	confess( (caller(0))[3] . ": missing required argument old_g. Dying" ) unless $old_g && defined $new_g;
+
+	if ($no_term_links)
+	{	$new_g->add_term($old_g->noderef($_)) foreach @{$old_g->terms};
+	}
+	else
+	{	# add all the relations from the other graph
+		$new_g = add_all_relations_to_graph($args);
+
+		foreach (@{$old_g->terms})
+		{	$new_g->add_term($old_g->noderef($_));
+			if ($old_g->get_outgoing_links($_))
+			{	foreach (@{$old_g->get_outgoing_links($_)})
+				{	# add the target term if it isn't there
+					$new_g->add_term($old_g->noderef($_->target)) if ! $new_g->get_term($_->target);
+					$new_g->add_link( new GOBO::LinkStatement(
+						node => $new_g->noderef($_->node),
+						relation => $new_g->noderef($_->relation),
+						target => $new_g->noderef($_->target)
+					) );
+				}
+			}
+		}
+	}
+	
+	return $new_g;
+}
+
+
+=head2 add_extra_stuff_to_graph
+
+input:  old_g => old Graph object
+        new_g => new Graph object (created if does not exist)
+
+output: new graph with various attributes from the old graph added 
+
+=cut
+
+sub add_extra_stuff_to_graph {
+	my $args = shift;
+	my $old_g = $args->{old_g};
+	my $new_g = $args->{new_g};
+
+	$new_g = new GOBO::Graph if ! $new_g;
+
+	confess( (caller(0))[3] . ": missing required arguments. Dying" ) unless $old_g && defined $new_g;
+
+	foreach my $attrib qw( version source date comment declared_subsets property_value_map )
+	{	$new_g->$attrib( $old_g->$attrib ) if $old_g->$attrib;
+	}
+	return $new_g;
+}
+
+
+=head2 add_nodes_and_links_to_graph
+
+input:  graph_data => data hash with nodes and relations specified as
+             { node_id }{ relation_id }{ target_id }
+        old_g      => old graph, containing nodes and relations specified in the data hash
+        new_g      => preferably the new graph, containing relations
+                      (created if does not exist)
+
+output: new graph, containing all the nodes and relations specified in 
+
+=cut
+
+sub add_nodes_and_links_to_graph {
+	my $args = shift;
+	my $graph_data = $args->{graph_data};
+	my $old_g = $args->{old_g};
+	my $new_g = $args->{new_g};
+
+	$new_g = new GOBO::Graph if ! $new_g;
+
+	confess( (caller(0))[3] . ": missing required arguments. Dying" ) unless $old_g && defined $new_g && $graph_data;
+
+	## add the relations to the graph if absent
+	if (! @{$new_g->relations} )
+	{	$new_g = add_all_relations_to_graph($args);
+	}
+
+	# add the nodes to the graph
+	foreach my $n ( keys %$graph_data )
+	{	# add the nodes to the graph
+		$new_g->add_term( $old_g->noderef( $n ) ) if ! $new_g->get_term($n);
+
+		foreach my $r ( keys %{$graph_data->{$n}} )
+		{	foreach my $t ( keys %{$graph_data->{$n}{$r}} )
+			{	$new_g->add_term( $old_g->noderef( $t ) ) if ! $new_g->get_term($t);
+				$new_g->add_link( new GOBO::LinkStatement(
+					node => $new_g->noderef($n),
+					relation => $new_g->noderef($r),
+					target => $new_g->noderef($t)
+				) );
+			}
+		}
+	}
+	return $new_g;
+}
+
+
 1;
